<?php

namespace Drupal\pki_ra\Processors;

use Drupal\Component\Utility\Html;
use Drupal\Component\Utility\UrlHelper;

class PKIRACertificationAuthority {
  const CERTIFICATES = 'certificates';
  const AUTHENTICATION_DELIMITER = ': ';

  protected $url;
  protected $authentication;
  protected $status;
  protected $headers;
  protected $response;

  /**
   * @todo Verify that the CA is a trusted host using Symfony\Component\HttpFoundation\Request.
   */
  public function __construct($url, $authentication = '') {
    if (!UrlHelper::isValid($url, TRUE)) {
      throw new \Exception('The certificate authority (CA) URL was not entered on the PKI Registration Authority settings page.');
    }

    if (!empty($authentication) && !self::authenticationIsProperlyFormatted($authentication)) {
      throw new \Exception('The certificate authority (CA) authentication header was not formatted correctly.');
    }

    $this->url = $url;
    $this->authentication = $authentication;
  }

  public static function authenticationIsProperlyFormatted($authentication) {
    if ((Html::escape($authentication) == $authentication) &&
        (count(explode(self::AUTHENTICATION_DELIMITER, $authentication)) == 2)) {
      return TRUE;
    }
    return FALSE;
  }

  public function signCertificate($csr, $profile, $email) {
    $target = $this->url . '/' . self::CERTIFICATES;

    try {
      $request = \Drupal::httpClient()->post($target, [
        'json' => [
          'csr' => $csr,
          'profile' => $profile,
          'email' => $email,
        ],
        'headers' => $this->getHeaders(),
      ]);
    }
    catch (Exception $e) {
      watchdog_exception('pki_ra', $e->getMessage());
    }

    $this->status = $request->getStatusCode();
    $this->headers = $request->getHeaders();
    $this->response = json_decode((string) $request->getBody(), TRUE);

    return $this;
  }

  protected function getHeaders() {
    if (empty($this->authentication)) {
      return [];
    }
    return [$this->getAuthenticationHeaderKey() => $this->getAuthenticationHeaderValue()];
  }

  protected function getAuthenticationHeaderKey() {
    return $this->getAuthenticationHeaderElement(0);
  }

  protected function getAuthenticationHeaderValue() {
    return $this->getAuthenticationHeaderElement(1);
  }

  protected function getAuthenticationHeaderElement($elementId) {
    return trim(explode(self::AUTHENTICATION_DELIMITER, $this->authentication)[$elementId]);
  }

  public function getCertificateSigningResponse() {
    return $this->response;
  }

  public function getCertificateSigningHeaders() {
    return $this->headers;
  }

  public function getCertificateSigningStatus() {
    return $this->status;
  }
}
